//**************************************************************************
// In diesem File finden Sie Musterlösungen zu einigen Aufgabenstellungen aus 
// Aufgabenblatt6. Diese sollen Ihnen helfen die geforderte Funktionalität 
// in Aufgabenblatt7 zu realisieren.
//**************************************************************************
class Spielfeld {
    private int[][] board;

    public Spielfeld() {
        board = new int[6][7];
    }
    
    public Spielfeld(Spielfeld base) {
        this();
        for (int r = 0; r <= 5; r++){
            for (int c = 0; c <= 6; c++) {
                board[r][c] = base.board[r][c];
            }
        }
    }
    
    public int feld(int reihe, int spalte) {
        return board[reihe][spalte];
    }
    
    public void spielstand() {
        for(int r = 5; r >= 0; r--){
            System.out.print("|");
            for(int cell : board[r]){
                switch(cell){
                    case 0:
                        System.out.print(" ");
                        break;
                    case 1:
                        System.out.print("x");
                        break;
                    case 2:
                        System.out.print("o");
                        break;
                }
            }
            System.out.println("|");
        }
        System.out.println("+-------+");
    }
    
    public Spielfeld zug(int spieler, int spalte) {
        if (spalte < 0 || spalte > 6){
            return null;
        }
        if (board[5][spalte] != 0){
            return null;
        }
        Spielfeld s = new Spielfeld(this);
        int r = 0;
        while(s.board[r][spalte] != 0){
            r++;
        }
        s.board[r][spalte] = spieler;
        return s;
    }
    
    public boolean sieg(int spieler) {
        // first, check 4-in-a-row:
        for (int r = 0; r <= 5; r++) {
            int cont = 0;
            for (int c = 0; c <= 6; c++) {
                if (board[r][c] == spieler){
                    cont++;
                }
                else {
                    cont = 0;
                }
                if (cont == 4) {
                    return true;
                }
            }
        }
        
        // now, check 4-in-a-column:
        for (int c = 0; c <= 6; c++) {
            int cont = 0;
            for (int r = 0; r <= 5; r++) {
                if (board[r][c] == spieler){
                    cont++;
                }
                else{
                    cont = 0;
                }
                if (cont == 4) {
                    return true;
                }
            }
        }
        
        // then, check 4-in-a-positive-diagonal:
        for (int d = -2; d <= 3; d++) {
            int cont = 0;
            for (int r = 0; r <= 5; r++) {
                int c = r + d;
                if (c >= 0 && c <= 6) {
                    if (board[r][c] == spieler) {
                        cont++;
                    } else {
                        cont = 0;
                    }
                    if (cont == 4) {
                        return true;
                    }
                }
            }
        }
        
        // finally, check 4-in-a-negative-diagonal:
        for (int d = 3; d <= 8; d++) {
            int cont = 0;
            for (int r = 5; r >= 0; r--) {
                int c = d - r;
                if (c >= 0 && c <= 6) {
                    if (board[r][c] == spieler) {
                        cont++;
                    } else {
                        cont = 0;
                    }
                    if (cont == 4) {
                        return true;
                    }
                }
            }
        }
        return false;
    }
    
    public int wert1(int spieler) {
        int result = 0;
        for (int r = 0; r <= 5; r++) {
            for (int c = 0; c <= 6; c++) {
                for (int len = 2, weight = 1; len <= 4; len++, weight *= 100) {
                    boolean isRow = true, isColumn = true, isDiagP = true, isDiagN = true;
                    for (int i = 0; i < len; i++) {
                        if (c + i > 6 || board[r][c + i] != spieler) isRow = false;
                        if (r + i > 5 || board[r + i][c] != spieler) isColumn = false;
                        if (r + i > 5 || c + i > 6 || board[r + i][c + i] != spieler) isDiagP = false;
                        if (r - i < 0 || c + i > 6 || board[r - i][c + i] != spieler) isDiagN = false;
                    }
                    if (isRow) result += weight;
                    if (isColumn) result += weight;
                    if (isDiagP) result += weight;
                    if (isDiagN) result += weight;
                }
            }
        }
        return result;
    }
    
    public int wert(int spieler) {
        int opponent = spieler == 1 ? 2 : 1;
        return wert1(spieler) - wert1(opponent);
    }
    
    public int negamax(int spieler, int tiefe) {
        if(tiefe == 0){
            return wert(spieler);
        }
        int opponent = spieler == 1 ? 2 : 1;
        int bestMaxValue = Integer.MIN_VALUE;
        for(int i = 0; i <= 6; i++){
            Spielfeld afterZug = zug(spieler, i);
            if(afterZug != null){
                int currentMaxValue;
                if(afterZug.sieg(spieler)){
                    currentMaxValue = Integer.MAX_VALUE;
                }
                else {
                    currentMaxValue = -(afterZug.negamax(opponent, tiefe - 1));
                }
                if(currentMaxValue > bestMaxValue){
                    bestMaxValue = currentMaxValue;
                }
            }
        }
        return bestMaxValue;
    }
    
    public int bester(int spieler, int tiefe) {
        int bestIndex = -1;
        int opponent = spieler == 1 ? 2 : 1;
        int bestMaxValue = Integer.MIN_VALUE;
        for(int i = 0; i <= 6; i++){
            Spielfeld afterZug = zug(spieler, i);
            if(afterZug != null){
                if(afterZug.sieg(spieler)){
                    return i;
                }
                int currentMaxValue = -(afterZug.negamax(opponent,tiefe));
                if(currentMaxValue > bestMaxValue){
                    bestMaxValue = currentMaxValue;
                    bestIndex = i;
                }
            }
        }
        return bestIndex;
    }
}

class Viergewinnt {
    public static void spiel() {
		//...
    }
    
    public static void spiel1(int tiefe) {
		//...   
    }
}

